c      SUBROUTINE FEMCG3(NODE,NSIZE,SYK,FORCE,DISP,INDEX2,INDEX5,NSUM,
c     *                  IWK,FCON,DCON,EPS,MITR,ERR,NITR,CONST,ISEL,
c     *                  DIAG,DD,T1,T2,T3,T4)
      SUBROUTINE FEMCG3
c
c        FEMCG3, DSCALE, PCG, ICCG1, ICCG2, CALAX
c               COPYRIGHT : T.MIYOSHI, FEB. 28 1991
c
c               SOLVES STIFFNESS EQUATION FOR THREE DIMENSIONAL
c               STRUCTURAL ANALYSIS BY SCG/ICCG METHOD.
c
c        INPUT - -
c               NTN       I *4  : NUMBER OF NODES
c               NSIZE      I *4  : SIZE OF ARRAY SYK
c               SYK()      R *8  : 1-DIM. ARRAY CONTAINING THE
c                                  STIFFNESS MATRIX
c               F()    R *8  : FORCES
c               INDEX2()   I *4  : ADDRESS OF PIVOT IN 1-DIM. ARRAY SYK
c               INDEX5()   I *4  : ADDRESS OF NON-ZERO ELEMENT IN 
c                                  1-DIM. ARRAY SYK
c               NSUM()     I *4  : ACCUMULATION OF NUMBER OF NON-ZERO
c                                  ELEMENTS
c               FCON       R *8  : INITIAL VALUE OF CONSTANT FOR
c                                  INCOMPLETE CHOLESKY DECOMPOSITION
c               DCON       R *8  : INCREMENT OF CONSTANT FOR
c                                  INCOMPLETE CHOLESKY DECOMPOSITION
c               EPS        R *8  : CONVERGENCE CRITERIA
c               MITR       I *4  : MAXIMUM NUMBER OF ITERATIONS
c               ISEL       I *4  : = 0, SELECTS ICCG METHOD
c                                  = 1, SELECTS SCG METHOD
c        OUTPUT - -
c               U()     R *8  : DISPLACEMENTS
c               NITR       I *4  : NUMBER OF ITERATIONS
c               ERR        R *8  : RESIDUAL ERROR
c               CONST      R *8  : CONSTANT FOR INCOMPLETE CHOLESKY
c                                  DECOMPOSITION
c        WORK - -
c               IWK()      I *4  : WORK AREA
c               DIAG()     R *8  : WORK AREA FOR DIAGONAL ELEMENTS
c               DD()       R *8  : WORK AREA FOR DECOMPOSITION
c               T1(),T2(),T3(),T4()
c                          R *8  : WORK AREA FOR CG ITERATION
c        NOTICE - -
c               NUMBER OF DEGREES OF FREEDOM = NODE*3
c
      use fem_arrays
      implicit integer (i-n)
      implicit double precision (a-h,o-z)
c
c      DIMENSION SYK(NSYK),FORCE(*),DISP(*)
c      DIMENSION SYK(*),FORCE(*),DISP(*)
c      DIMENSION DIAG(*),DD(*),T1(*),T2(*),T3(*),T4(*)
c      DIMENSION INDEX2(*),INDEX5(*),NSUM(*),IWK(*)
c
c      NDOF=NODE*3
      NDOF=NTN*3
      IF(INDEX2(NDOF).GT.NSYK) THEN
        WRITE(6,9000) INDEX2(NDOF)
 9000   FORMAT(/2X,'(SUBR. FEMCG3) INSUFFICIENT ARRAY SIZE(>=',
     *         I15,')'/)
        STOP
      ENDIF
c
      IF(ISEL.NE.0.AND.ISEL.NE.1) ISEL=0
      IF(EPS.EQ.0.0) EPS=1.0E-4
      IF(FCON.EQ.0.0) FCON=2.0
      IF(DCON.EQ.0.0) DCON=0.1
      IF(MITR.EQ.0) MITR=NDOF
c      CALL DSCALE(NODE,SYK,DIAG,FORCE,INDEX2,INDEX5,NSUM,IWK)
c      CALL PCG(NODE,SYK,FORCE,DISP,INDEX2,INDEX5,NSUM,IWK,FCON,DCON,
c     *         EPS,MITR,ERR,NITR,CONST,DIAG,DD,T1,T2,T3,T4,ISEL)
      CALL DSCALE
      CALL PCG
c
      DO 100 I=1,NTN
        U(3*I-2)=U(3*I-2)*DIAG(3*I-2)
        U(3*I-1)=U(3*I-1)*DIAG(3*I-1)
        U(3*I  )=U(3*I  )*DIAG(3*I  )
  100 CONTINUE
c
c
      RETURN
      END
c--------------------------------------------------------------------
c      SUBROUTINE PCG(NODE,SYK,B,X,INDEX2,INDEX5,NSUM,IWK,FCON,DCON,
c     *               EPS,MITR,ERR,NITR,CONST,DIAG,DD,T1,T2,T3,T4,ISEL)
      SUBROUTINE PCG
c
c        PCG(CALLED FROM FEMCG3)
c               COPYRIGHT : T.MIYOSHI, FEB. 28 1991
c
c               SOLVES STIFFNESS EQUATION FOR THREE DIMENSIONAL
c               STRUCTURAL ANALYSIS BY SCG/ICCG METHOD.
c
c        INPUT - -
c               NTN       I *4  : NUMBER OF NODES
c               SYK()      R *8  : 1-DIM. ARRAY CONTAINING THE
c                                  STIFFNESS MATRIX
c               B_CG()        R *8  : RIGHT HAND SIDE VECTOR
c               INDEX2()   I *4  : ADDRESS OF PIVOT IN 1-DIM. ARRAY SYK
c               INDEX5()   I *4  : ADDRESS OF NON-ZERO ELEMENT IN
c                                  1-DIM. ARRAY SYK
c               NSUM()     I *4  : ACCUMULATION OF NUMBER OF NON-ZERO
c                                  ELEMENTS
c               FCON       R *8  : INITIAL VALUE OF CONSTANT FOR
c                                  INCOMPLETE CHOLESKY DECOMPOSITION
c               DCON       R *8  : INCREMENT OF CONSTANT FOR
c                                  INCOMPLETE CHOLESKY DECOMPOSITION
c               EPS        R *8  : CONVERGENCE CRITERIA
c               MITR       I *4  : MAXIMUM NUMBER OF ITERATIONS
c               ISEL       I *4  : = 0, SELECTS ICCG METHOD
c                                  = 1, SELECTS SCG METHOD
c        OUTPUT - -
c               X_CG()        R *8  : SOLUTION
c               NITR       I *4  : NUMBER OF ITERATIONS
c               ERR        R *8  : RESIDUAL ERROR
c               CONST      R *8  : CONSTANT FOR INCOMPLETE CHOLESKY
c                                  DECOMPOSITION
c        WORK - -
c               IWK()      I *4  : WORK AREA
c               DD()       R *8  : WORK AREA FOR DECOMPOSITION
c               DIAG()     R *8  : WORK AREA FOR DIAGONAL ELEMENTS
c               T1(),T2(),T3(),T4()
c                          R *8  : WORK AREA FOR CG ITERATION
c        NOTICE - -
c               NUMBER OF DEGREES OF FREEDOM = NTN*3
c
      use fem_arrays
      implicit integer (i-n)
      implicit double precision (a-h,o-z)
c
c      IF(ISEL.EQ.0) CALL ICCG1(NODE,SYK,INDEX2,INDEX5,NSUM,IWK,DD,
c     *                         FCON,DCON,CONST)
      IF(ISEL.EQ.0) CALL ICCG1
c
c      CALL CALAX(NODE,SYK,X,T1,INDEX2,INDEX5,NSUM)
cccc modified 00-05-08
c      CALL CALAX(X_CG,T1)
      CALL CALAX(U,T1)
c
      BNR1=0.0D0
      BNR2=0.0D0
      BNR3=0.0D0
      DO 200 I=1,NTN
c        BNR1=BNR1+B_CG(3*I-2)**2
c        BNR2=BNR2+B_CG(3*I-1)**2
c        BNR3=BNR3+B_CG(3*I  )**2
c        T1(3*I-2)=B_CG(3*I-2)-T1(3*I-2)
c        T1(3*I-1)=B_CG(3*I-1)-T1(3*I-1)
c        T1(3*I  )=B_CG(3*I  )-T1(3*I  )
cccc modified 00-05-08
        BNR1=BNR1+F(3*I-2)**2
        BNR2=BNR2+F(3*I-1)**2
        BNR3=BNR3+F(3*I  )**2
        T1(3*I-2)=F(3*I-2)-T1(3*I-2)
        T1(3*I-1)=F(3*I-1)-T1(3*I-1)
        T1(3*I  )=F(3*I  )-T1(3*I  )
  200 CONTINUE
      BNR=BNR1+BNR2+BNR3
c
      IF(ISEL.EQ.0) THEN
c        CALL ICCG2(NODE,SYK,DD,T1,T2,INDEX2,INDEX5,NSUM,IWK)
        CALL ICCG2(T1,T2)
      ELSE
        DO 250 I=1,NTN
          T2(3*I-2)=1.0D0/DIAG(3*I-2)*T1(3*I-2)
          T2(3*I-1)=1.0D0/DIAG(3*I-1)*T1(3*I-1)
          T2(3*I  )=1.0D0/DIAG(3*I  )*T1(3*I  )
  250   CONTINUE
      ENDIF
*
      R11=0.0D0
      R12=0.0D0
      R13=0.0D0
      DO 300 I=1,NTN
        R11=R11+T1(3*I-2)*T2(3*I-2)
        R12=R12+T1(3*I-1)*T2(3*I-1)
        R13=R13+T1(3*I  )*T2(3*I  )
  300 CONTINUE
      R1=R11+R12+R13
c
      DO 400 L=1,MITR
      WRITE(6,*)'STEP & ERROR',L,ERR
c
c        CALL CALAX(NODE,SYK,T2,T4,INDEX2,INDEX5,NSUM)
        CALL CALAX(T2,T4)
c
        R21=0.0D0
        R22=0.0D0
        R23=0.0D0
        DO 410 I=1,NTN
          R21=R21+T2(3*I-2)*T4(3*I-2)
          R22=R22+T2(3*I-1)*T4(3*I-1)
          R23=R23+T2(3*I  )*T4(3*I  )
  410   CONTINUE
        R2=R21+R22+R23
        ALP=R1/R2
        RNR1=0.0D0
        RNR2=0.0D0
        RNR3=0.0D0
        DO 420 I=1,NTN
cccc modified 00-05-08
c          X_CG(3*I-2)=X_CG(3*I-2)+ALP*T2(3*I-2)
c          X_CG(3*I-1)=X_CG(3*I-1)+ALP*T2(3*I-1)
c          X_CG(3*I  )=X_CG(3*I  )+ALP*T2(3*I  )
          U(3*I-2)=U(3*I-2)+ALP*T2(3*I-2)
          U(3*I-1)=U(3*I-1)+ALP*T2(3*I-1)
          U(3*I  )=U(3*I  )+ALP*T2(3*I  )
          T1(3*I-2)=T1(3*I-2)-ALP*T4(3*I-2)
          T1(3*I-1)=T1(3*I-1)-ALP*T4(3*I-1)
          T1(3*I  )=T1(3*I  )-ALP*T4(3*I  )
          RNR1=RNR1+T1(3*I-2)**2
          RNR2=RNR2+T1(3*I-1)**2
          RNR3=RNR3+T1(3*I  )**2
  420   CONTINUE
        RNR=RNR1+RNR2+RNR3
        ERR=SQRT(RNR/BNR)
        IF(ERR.LT.EPS) GO TO 900
c
        IF(ISEL.EQ.0) THEN
c          CALL ICCG2(NODE,SYK,DD,T1,T3,INDEX2,INDEX5,NSUM,IWK)
          CALL ICCG2(T1,T3)
        ELSE
          DO 425 I=1,NTN
            T3(3*I-2)=1.0D0/DIAG(3*I-2)*T1(3*I-2)
            T3(3*I-1)=1.0D0/DIAG(3*I-1)*T1(3*I-1)
            T3(3*I  )=1.0D0/DIAG(3*I  )*T1(3*I  )
  425     CONTINUE
        ENDIF
c
        R11=0.0D0
        R12=0.0D0
        R13=0.0D0
        R31=0.0D0
        R32=0.0D0
        R33=0.0D0
        DO 430 I=1,NTN
          R11=R11+T1(3*I-2)*T3(3*I-2)
          R12=R12+T1(3*I-1)*T3(3*I-1)
          R13=R13+T1(3*I  )*T3(3*I  )
          R31=R31+T3(3*I-2)*T4(3*I-2)
          R32=R32+T3(3*I-1)*T4(3*I-1)
          R33=R33+T3(3*I  )*T4(3*I  )
  430   CONTINUE
        R1=R11+R12+R13
        R3=R31+R32+R33
        BETA=-R3/R2
        DO 440 I=1,NTN
          T2(3*I-2)=T3(3*I-2)+BETA*T2(3*I-2)
          T2(3*I-1)=T3(3*I-1)+BETA*T2(3*I-1)
          T2(3*I  )=T3(3*I  )+BETA*T2(3*I  )
  440   CONTINUE
  400 CONTINUE
c      WRITE(6,*)'STEP & ERROR',L,ERR
c
      L=MITR
      IF(L.EQ.MITR) THEN
        WRITE(*,*) '  (SUBR. PCG) MAXIMUM NUMBER OF ITERATIONS REACHED'
        STOP
      ENDIF
  900 CONTINUE
      NITR=L
c
c
      RETURN
      END
c------------------------------------------------------------------------
c      SUBROUTINE ICCG1(NODE,SYK,INDEX2,INDEX5,NSUM,IWK,DD,
c     *                 FCON,DCON,CONST)
      SUBROUTINE ICCG1
c
c        ICCG1(CALLED FROM PCG)
c               COPYRIGHT : T.MIYOSHI, FEB. 28 1991
c
c               SOLVES STIFFNESS EQUATION FOR THREE DIMENSIONAL
c               STRUCTURAL ANALYSIS BY SCG/ICCG METHOD.
c
c        INPUT - -
c               NTN       I *4  : NUMBER OF NTN
c               SYK()      R *8  : 1-DIM. ARRAY CONTAINING THE
c                                  STIFFNESS MATRIX
c               INDEX2()   I *4  : ADDRESS OF PIVOT IN 1-DIM. ARRAY SYK
c               INDEX5()   I *4  : ADDRESS OF NON-ZERO ELEMENT IN
c                                  1-DIM. ARRAY SYK
c               NSUM()     I *4  : ACCUMULATION OF NUMBER OF NON-ZERO
c                                  ELEMENTS
c               FCON       R *8  : INITIAL VALUE OF CONSTANT FOR
c                                  INCOMPLETE CHOLESKY DECOMPOSITION
c               DCON       R *8  : INCREMENT OF CONSTANT FOR
c                                  INCOMPLETE CHOLESKY DECOMPOSITION
c        OUTPUT - -
c               DD()       R *8  : INVERSE OF [D] MATRIX
c               CONST      R *8  : CONSTANT FOR INCOMPLETE CHOLESKY
c                                  DECOMPOSITION
c        WORK - -
c               IWK()      I *4  : WORK AREA
c        NOTICE - -
c               NUMBER OF DEGREES OF FREEDOM = NTN*3
*
      use fem_arrays
      implicit integer (i-n)
      implicit double precision (a-h,o-z)
*
      CONST=FCON
*
 7000 CONTINUE
c      WRITE(6,*)'WEIGHT=',CONST
*
      DD(1)=1.0D0/SYK(1)
      DD(2)=1.0D0/(CONST*SYK(3)-SYK(2)**2*DD(1))
      DD(3)=1.0D0/(CONST*SYK(6)-SYK(4)**2*DD(1)-SYK(5)**2*DD(2))
      DO 200 I=2,NTN
        IE=3*(NSUM(I)-NSUM(I-1))
c        write(6,*)I,IE/3
        S1=0.0D0
        S2=0.0D0
        S3=0.0D0
        IF(IE.EQ.0) GO TO 120
        JE=NSUM(I)-NSUM(I-1)
        DO 150 J=1,JE
          J3=3*J
          JJ=INDEX5(NSUM(I-1)+J)*3
          IWK(J3-2)=JJ-2
          IWK(J3-1)=JJ-1
          IWK(J3  )=JJ
  150   CONTINUE
        DO 100 J=1,IE
          S1=S1+SYK(INDEX2(3*I-3)+J)**2*DD(IWK(J))
          S2=S2+SYK(INDEX2(3*I-2)+J)**2*DD(IWK(J))
          S3=S3+SYK(INDEX2(3*I-1)+J)**2*DD(IWK(J))
  100   CONTINUE
  120   CONTINUE
        DD(3*I-2)=1.0D0/(CONST*SYK(INDEX2(3*I-2))-S1)
        DD(3*I-1)=1.0D0/(CONST*SYK(INDEX2(3*I-1))-S2
     *                  -SYK(INDEX2(3*I-1)-1)**2*DD(3*I-2))
        DD(3*I  )=1.0D0/(CONST*SYK(INDEX2(3*I ))-S3
     *                  -SYK(INDEX2(3*I  )-2)**2*DD(3*I-2)
     *                  -SYK(INDEX2(3*I  )-1)**2*DD(3*I-1))
  200 CONTINUE
*
c      NTFR=NODE*3
      NTFR=NTN*3
      DO 300 I=1,NTFR
c         write(6,*)'DD=',I,DD(I)
        IF(DD(I).LT.1.0E-10) THEN
          CONST=CONST+DCON
          GO TO 7000
        ENDIF
  300 CONTINUE
*
*
      RETURN
      END
c--------------------------------------------------------------------
c      SUBROUTINE ICCG2(NODE,SYK,DD,B,X,INDEX2,INDEX5,NSUM,IWK)
      SUBROUTINE ICCG2(B_CG,X_CG)
*
c        ICCG2(CALLED FROM PCG)
c               COPYRIGHT : T.MIYOSHI, FEB. 28 1991
*
c               SOLVES STIFFNESS EQUATION FOR THREE DIMENSIONAL
c               STRUCTURAL ANALYSIS BY SCG/ICCG METHOD.
*
c        INPUT - -
c               NTN       I *4  : NUMBER OF NODES
c               SYK()      R *8  : 1-DIM. ARRAY CONTAINING THE
c                                  STIFFNESS MATRIX
c               B_CG()        R *8  : RIGHT HAND SIDE VECTOR
c               INDEX2()   I *4  : ADDRESS OF PIVOT IN 1-DIM. ARRAY SYK
c               INDEX5()   I *4  : ADDRESS OF NON-ZERO ELEMENT IN
c                                  1-DIM. ARRAY SYK
c               NSUM()     I *4  : ACCUMULATION OF NUMBER OF NON-ZERO
c                                  ELEMENTS
c               DD()       R *8  : INVERSE OF [D] MATRIX CALCULATED
c                                  IN SUBROUTINE ICCG1
c        OUTPUT - -
c               X_CG()        R *8  : SOLUTION
c        WORK - -
c               IWK()      I *4  : WORK AREA
c        NOTICE - -
c               NUMBER OF DEGREES OF FREEDOM = NTN*3
*
      use fem_arrays
      implicit integer (i-n)
      implicit double precision (a-h,o-z)
      dimension b_cg(ntf),x_cg(ntf)
c
      X_CG(1)=B_CG(1)
      X_CG(2)=B_CG(2)-SYK(2)*DD(1)*X_CG(1)
      X_CG(3)=B_CG(3)-SYK(4)*DD(1)*X_CG(1)-SYK(5)*DD(2)*X_CG(2)
      DO 200 I=2,NTN
        JE=NSUM(I)-NSUM(I-1)
        JE3=3*JE
        S1=0.0D0
        S2=0.0D0
        S3=0.0D0
        IF(JE.EQ.0) GO TO 120
        DO 150 J=1,JE
          J3=J*3
          JJ=INDEX5(NSUM(I-1)+J)*3
          IWK(J3-2)=JJ-2
          IWK(J3-1)=JJ-1
          IWK(J3  )=JJ
  150   CONTINUE
        DO 100 J=1,JE3
          IJK=IWK(J)
          S1=S1+SYK(INDEX2(3*I-3)+J)*DD(IJK)*X_CG(IJK)
          S2=S2+SYK(INDEX2(3*I-2)+J)*DD(IJK)*X_CG(IJK)
          S3=S3+SYK(INDEX2(3*I-1)+J)*DD(IJK)*X_CG(IJK)
  100   CONTINUE
  120   CONTINUE
       X_CG(3*I-2)=B_CG(3*I-2)-S1
       X_CG(3*I-1)=B_CG(3*I-1)-S2
     *                 -SYK(INDEX2(3*I-1)-1)*DD(3*I-2)*X_CG(3*I-2)
       X_CG(3*I  )=B_CG(3*I  )-S3
     *                 -SYK(INDEX2(3*I  )-2)*DD(3*I-2)*X_CG(3*I-2)
     *                 -SYK(INDEX2(3*I  )-1)*DD(3*I-1)*X_CG(3*I-1)
  200 CONTINUE
      NTFR=NTN*3
      X_CG(NTFR)=X_CG(NTFR)*DD(NTFR)
      X_CG(NTFR-1)=(X_CG(NTFR-1)
     *                 -SYK(INDEX2(NTFR  )-1)*X_CG(NTFR  ))*DD(NTFR-1)
      X_CG(NTFR-2)=(X_CG(NTFR-2)
     *                 -SYK(INDEX2(NTFR  )-2)*X_CG(NTFR  )
     *                 -SYK(INDEX2(NTFR-1)-1)*X_CG(NTFR-1))*DD(NTFR-2)
*
      DO 400 I=NTN-1,1,-1
        JE=NSUM(I+1)-NSUM(I)
        JE3=3*JE
        IF(JE.EQ.0) GO TO 220
        DO 350 J=1,JE
          J3=3*J
          JJ=INDEX5(NSUM(I)+J)*3
          IWK(J3-2)=JJ-2
          IWK(J3-1)=JJ-1
          IWK(J3  )=JJ
  350   CONTINUE
        DO 300 J=1,JE3
       X_CG(IWK(J))=X_CG(IWK(J))-SYK(INDEX2(3*I  )+J)*X_CG(3*I+1)
     *                          -SYK(INDEX2(3*I+1)+J)*X_CG(3*I+2)
     *                          -SYK(INDEX2(3*I+2)+J)*X_CG(3*I+3)
  300   CONTINUE
  220   CONTINUE
        X_CG(3*I)=X_CG(3*I)*DD(3*I)
        X_CG(3*I-1)=(X_CG(3*I-1)
     *                    -SYK(INDEX2(3*I)-1)*X_CG(3*I  ))*DD(3*I-1)
        X_CG(3*I-2)=(X_CG(3*I-2)
     *                    -SYK(INDEX2(3*I)-2)*X_CG(3*I  )
     *                    -SYK(INDEX2(3*I-1)-1)*X_CG(3*I-1))*DD(3*I-2)
  400 CONTINUE
*
*
      RETURN
      END
c--------------------------------------------------------------------
c      SUBROUTINE CALAX(NODE,SYK,X,Y,INDEX2,INDEX5,NSUM)
      SUBROUTINE CALAX(X_CG,Y_CG)
*
c        CALAX(CALLED FROM PCG)
c               COPYRIGHT : T.MIYOSHI, FEB. 28 1991
*
c               SOLVES STIFFNESS EQUATION FOR THREE DIMENSIONAL
c               STRUCTURAL ANALYSIS BY SCG/ICCG METHOD.
*
c        INPUT - -
c               NTN       I *4  : NUMBER OF NODES
c               SYK()      R *8  : 1-DIM. ARRAY CONTAINING THE
c                                  STIFFNESS MATRIX
c               X_CG()        R *8  : MULTIPLIED VECTOR
c               INDEX2()   I *4  : ADDRESS OF PIVOT IN 1-DIM. ARRAY SYK
c               INDEX5()   I *4  : ADDRESS OF NON-ZERO ELEMENT IN
c                                  1-DIM. ARRAY SYK
c               NSUM()     I *4  : ACCUMULATION OF NUMBER OF NON-ZERO
c                                  ELEMENTS
c        OUTPUT - -
c               Y_CG()        R *8  : SOLUTION(A*X)
c        NOTICE - -
c               NUMBER OF DEGREES OF FREEDOM = NTN*3
*
      use fem_arrays
      implicit integer (i-n)
      implicit double precision (a-h,o-z)
      dimension x_cg(ntf),y_cg(ntf)
*
      DO 100 I=1,NTN
       N1=3*I-2
       N2=3*I-1
       N3=3*I
       Y_CG(N1)=SYK(INDEX2(N1)  )*X_CG(N1)+SYK(INDEX2(N2)-1)*X_CG(N2)
     *                                 +SYK(INDEX2(N3)-2)*X_CG(N3)
       Y_CG(N2)=SYK(INDEX2(N2)-1)*X_CG(N1)+SYK(INDEX2(N2)  )*X_CG(N2)
     *                                 +SYK(INDEX2(N3)-1)*X_CG(N3)
       Y_CG(N3)=SYK(INDEX2(N3)-2)*X_CG(N1)+SYK(INDEX2(N3)-1)*X_CG(N2)
     *                                 +SYK(INDEX2(N3)  )*X_CG(N3)
  100 CONTINUE
*
      DO 120 I=2,NTN
        JE=NSUM(I)-NSUM(I-1)
        IF(JE.EQ.0) GO TO 120
        N1=3*I-2
        N2=3*I-1
        N3=3*I
        DO 110 J=1,JE
          M1=INDEX2(N1-1)+3*(J-1)+1
          M2=INDEX2(N2-1)+3*(J-1)+1
          M3=INDEX2(N3-1)+3*(J-1)+1
          LL=3*INDEX5(NSUM(I-1)+J)-2
         Y_CG(N1)=Y_CG(N1)+SYK(M1  )*X_CG(LL  )+SYK(M1+1)*X_CG(LL+1)
     *               +SYK(M1+2)*X_CG(LL+2)
         Y_CG(N2)=Y_CG(N2)+SYK(M2  )*X_CG(LL  )+SYK(M2+1)*X_CG(LL+1)
     *               +SYK(M2+2)*X_CG(LL+2)
         Y_CG(N3)=Y_CG(N3)+SYK(M3  )*X_CG(LL  )+SYK(M3+1)*X_CG(LL+1)
     *               +SYK(M3+2)*X_CG(LL+2)
  110   CONTINUE
  120 CONTINUE
*
      DO 140 I=2,NTN
        JE=NSUM(I)-NSUM(I-1)
        IF(JE.EQ.0) GO TO 140
        N1=3*I-2
        N2=3*I-1
        N3=3*I
        DO 130 J=1,JE
         M1=INDEX2(N1-1)+3*(J-1)+1
         M2=INDEX2(N2-1)+3*(J-1)+1
         M3=INDEX2(N3-1)+3*(J-1)+1
         LL=3*INDEX5(NSUM(I-1)+J)-2
         Y_CG(LL  )=Y_CG(LL  )+SYK(M1  )*X_CG(N1)+SYK(M2  )*X_CG(N2)
     *                   +SYK(M3  )*X_CG(N3)
         Y_CG(LL+1)=Y_CG(LL+1)+SYK(M1+1)*X_CG(N1)+SYK(M2+1)*X_CG(N2)
     *                   +SYK(M3+1)*X_CG(N3)
         Y_CG(LL+2)=Y_CG(LL+2)+SYK(M1+2)*X_CG(N1)+SYK(M2+2)*X_CG(N2)
     *                   +SYK(M3+2)*X_CG(N3)
  130   CONTINUE
  140 CONTINUE
*
*
      RETURN
      END
c--------------------------------------------------------------------
c      SUBROUTINE DSCALE(NODE,SYK,DIAG,FORCE,INDEX2,INDEX5,
c     *                  NSUM,IWK)
      SUBROUTINE DSCALE
*
c        DSCALE(CALLED FROM FEMCG3)
c               COPYRIGHT : T.MIYOSHI, FEB. 28 1991
*
c               SOLVES STIFFNESS EQUATION FOR THREE DIMENSIONAL
c               STRUCTURAL ANALYSIS BY SCG/ICCG METHOD.
*
c        INPUT - -
c               NTN       I *4  : NUMBER OF NODES
c               SYK()      R *8  : 1-DIM. ARRAY CONTAINING THE
c                                  STIFFNESS MATRIX
c               F()    R *8  : FORCES
c               INDEX2()   I *4  : ADDRESS OF PIVOT IN 1-DIM. ARRAY SYK
c               INDEX5()   I *4  : ADDRESS OF NON-ZERO ELEMENT IN
c                                  1-DIM. ARRAY SYK
c               NSUM()     I *4  : ACCUMULATION OF NUMBER OF NON-ZERO
c                                  ELEMENTS
c        OUTPUT - -
c               DIAG()     R *8  : DIAGONAL ELEMENTS
c        WORK - -
c               IWK()      I *4  : WORK AREA
c        NOTICE - -
c               NUMBER OF DEGREES OF FREEDOM = NTN*3
c
      use fem_arrays
      implicit integer (i-n)
      implicit double precision (a-h,o-z)
c
      DO 50 I=1,NTN
        DIAG(3*I-2)=1.0D0/SQRT(SYK(INDEX2(3*I-2)))
        DIAG(3*I-1)=1.0D0/SQRT(SYK(INDEX2(3*I-1)))
        DIAG(3*I  )=1.0D0/SQRT(SYK(INDEX2(3*I  )))
   50 CONTINUE
c
      DO 60 I=1,NTN
        SYK(INDEX2(3*I-2))=1.0D0
        SYK(INDEX2(3*I-1))=1.0D0
        SYK(INDEX2(3*I  ))=1.0D0
   60 CONTINUE
c
      SYK(2)=SYK(2)*DIAG(1)*DIAG(2)
      SYK(4)=SYK(4)*DIAG(1)*DIAG(3)
      SYK(5)=SYK(5)*DIAG(2)*DIAG(3)
      DO 70 I=2,NTN
        JE1=NSUM(I)-NSUM(I-1)
        JE2=3*JE1
        IF(JE1.EQ.0) GO TO 81
        DO 80 J=1,JE1
          J3=J*3
          JJ=INDEX5(NSUM(I-1)+J)*3
          IWK(J3-2)=JJ-2
          IWK(J3-1)=JJ-1
          IWK(J3  )=JJ
   80   CONTINUE
   81   CONTINUE
        II0=3*I-3
        II1=3*I-2
        II2=3*I-1
        II3=3*I
        IA0=INDEX2(II0)
        IA1=INDEX2(II1)
        IA2=INDEX2(II2)
        IA3=INDEX2(II3)
        IF(JE2.EQ.0) GO TO 91
        DO 90 J=1,JE2
          SYK(IA0+J)=SYK(IA0+J)*DIAG(II1)*DIAG(IWK(J))
          SYK(IA1+J)=SYK(IA1+J)*DIAG(II2)*DIAG(IWK(J))
          SYK(IA2+J)=SYK(IA2+J)*DIAG(II3)*DIAG(IWK(J))
   90   CONTINUE
   91   CONTINUE
        SYK(IA2-1)=SYK(IA2-1)*DIAG(II2)*DIAG(II1)
        SYK(IA3-2)=SYK(IA3-2)*DIAG(II3)*DIAG(II1)
        SYK(IA3-1)=SYK(IA3-1)*DIAG(II3)*DIAG(II2)
   70 CONTINUE
c
      DO 100 I=1,NTN
        F(3*I-2)=F(3*I-2)*DIAG(3*I-2)
        F(3*I-1)=F(3*I-1)*DIAG(3*I-1)
        F(3*I  )=F(3*I  )*DIAG(3*I  )
  100 CONTINUE
c
      RETURN
      END
