c   fortran90 by s.izumi 2000-05-0 8
      module fem_arrays
      implicit integer (i-n)
      implicit double precision (a-h,o-z)
c  idm   : dimension
c  inmte : max. number of a node in a element --> 8
c  inmnm : max. size of element stiffness matrix  inmte*idim
c  ird   : max. number of integration point in a element
c  icom  : number of stress component (2-dim->4 ,3dim->6)
      parameter(idim=3,inmte=20,inmnm=inmte*idim,icom=6,nfile=10)
c--------------------------------------------------------------
c for subroutie data
c  ngauss : order of gauss integration
c  npara  : analysis condition (0:plain stress,1:plain strain)
cccc  t      : thickness
c  ntn    : total number of nodes
c  nte    : total number of elements
c  ntf    : total number of freedoms
cccc  ie     : element number(variables)
c  ntfor  : total number of freedoms with initial load
c  ntdis  : total number of freedoms with initial displacement
c  nna    : the total number of integration points
c  mnum   : number of the kind of material
c  nb     : half band width
c--------------------------------------------------------------
        character*70 fnum(nfile)
        common /cha/ fnum
c
	common /nfm1/ ngauss,npara,ntn,nte,ntf,nna,mnum
	common /nfm2/ ntfor,ntdis,nb
	common /nfm3/ ie,na
cccc for iccg method
	common /nfm4/ nsyk
        common /nfm6/ MITR,NITR,ISEL
cccc for allocate
        common /nfm5/ nrd,nnmte,nnmnm
c
cccc for equation
c  naeq : number of equation
c     neqb : root freedom
c     nteq : number of copy freedom
c     neq  : copy freedom
	common /eq0/ naeq
	common /eq1/ neqb(5),nteq(5)
	common /eq2/ neq(5,100)
c
c  D(i,j) : elastic matrix (1-x,2-y,3-z,4-xy,5-xz,6-yz)
        common /dm1/ d(icom,icom)
        common /dm2/ xxi,eta,zeta,hnum
c----------------------------------------------------------------------
c  N    : shape function Ni
c  JINV : Inverse of Jacobian matrix
c  JM   : Jacobian matrix
c  DETJ   : det[J]
c----------------------------------------------------------------------
        double precision jinv(idim,idim),jm(idim,idim)
        common /dm3/ jinv,jm
        common /dm4/ detj
cccc for iccg method
        common /dm5/ FCON,DCON,EPS,ERR,CONST
c
c---------------------------------------------------------------------
c  NMTE(i) : number of a node in i-th element
c  NN(i,j) : node number of i-th local node of element j 
c  IFOR(i) : freedom number of i-th initial load point
c  IDIS(i) : freedom number of i-th initial disp. point
c  NGNG(i) : number of integration point in i-th element
c  NOM(i)  : material number of i-th element
c  NKINF(i) : kind of element 1--> infinite element
c  IECON(i)  : ELASTIC CHARACTER of i-th material
c  NEUS(i) : i : PROGRAM NODE NUMBER , NEUS(I) : USER ELEMENT NUMBER 
c  NNUS(i) : i : PROGRAM NODE NUMBER , NNUS(I) : USER NODE NUMBER 
c  NSGAV(i) :  summation number of global node i
      allocatable :: nmte(:),nn(:,:),ngng(:),nom(:),nkinf(:),neus(:)
      allocatable :: ifor(:),idis(:),iecon(:)
      allocatable :: nnus(:)
      allocatable :: nf(:)
      allocatable :: nsgav(:)

c  COORD(i,j) : i-th coordinate of node j 
c  ECON(i,j) : i-th elastic constants of j-th material 
c  FB(i)   : load value of i-th initial load point
c  UB(i)   : disp. value of i-th initial disp. point
c  EK(i,j) : element stiffness matrix 
c  GK(i,j) : global stiffness matrix  i: matrix size  j:half band width
c  F(i)    : force vector of node i
c  U(i)    : displacement value of i-th freedom
c  AXI(i)  : x-coordinate of i-th integration point ( -1 < AXI  < 1 )
c  AETA(i) : y-coordinate of i-th integration point ( -1 < AETA < 1 )
c  AZETA(i): z-coordinate of i-th integration point ( -1 < AZETA < 1 )
c  H(i)   : weight parameter of i-th integration point
c  DNDXI(i): dNi/dXI, DNDETA(i) : dNi/dETA
c  DXIF(i): dNi/dXI, DETIF(i) : dNi/dETA  : FOR INFINITE ELEMENT
c  B(i,j)  : elemental stiffness matrix
c  BM(i)   : For strain (B matrix for i-th integration point)
c  BT(i)   : transposed B(6,INMNM) matrix 
c  BTD(i)  : BT(INMNM,ICOM)*D(ICOM,ICOM)
c  C(i,j)  : BT(INMNM,ICOM)*D(ICOM,ICOM)*B(ICOM,INMNM) Bt*D*B
c  EP(i,j,k) : i-th component strain at j-th element
c                k-th local integration point
c  SG(i,j,k) : i-th component stress at j-th element
c                k-th local integration point
c  SGN(J,I) :  j:component i:node number(global)
c----------------------------------------------------------------------
c
      allocatable :: coord(:,:),econ(:,:)
      allocatable :: fb(:),ub(:)
      allocatable :: ek(:,:),gk(:,:),f(:),u(:)
      allocatable :: axi(:),aeta(:),azeta(:),h(:)
      allocatable :: dndxi(:),dndeta(:),dndzeta(:)
      allocatable :: dxif(:),detif(:),dzeif(:)
      allocatable :: b(:,:),bm(:,:,:)
      allocatable :: bt(:,:),btd(:,:),c(:,:)
      allocatable :: ep(:,:,:),sg(:,:,:)
      allocatable :: sgn(:,:),epn(:,:)
c  shn    : shape function Ni
c  shnif  : shape function Ni for infinite element
      allocatable :: shn(:)
      allocatable :: shnif(:)
c
ccc FOR ICCG METHOD
c NSMALL(I) : see data.f
c INDEX2,INDEX5,NSUM --> gyouretsu keisan sofrware p318
      allocatable :: nsmall(:),nsnode(:,:)
      allocatable :: index2(:),index5(:),nsum(:)
      allocatable :: syk(:)
      allocatable :: diag(:),dd(:),t1(:),t2(:),t3(:),t4(:)
      allocatable :: iwk(:)

ccc FOR CHOLESKY METHOD
c      DIMENSION GKC(IBN,IGK),DR(IGK),ZM(IGK)
c
      end module fem_arrays
c
c   MAIN PROGRAM   FORTRAN90 by S.izumi 2000-05-08
      program main
      use fem_arrays
      implicit integer (i-n)
      implicit double precision (a-h,o-z)
c**** file open
      integer index
      integer*4 k
      character*60 arg
      external getarg
      k=1
      call getarg ( k, arg )
c
      index = lnblnk(arg)
      if (index.eq.0) then
         arg = 'files'
         index = lnblnk(arg)
c      stop 'need file-directory'
      endif
c     
      do i = 1,nfile
      fnum(i)(1:70) = arg(1:index)
      enddo
c
      fnum(1)(index+1:70) = '/datacheck.dat'
      fnum(2)(index+1:70) = '/result.dat'
      fnum(3)(index+1:70) = '/input.dat'
      fnum(4)(index+1:70) = '/temp1.dat'
      fnum(5)(index+1:70) = '/temp2.dat'
c
      open(77,file=fnum(1),status='unknown')
      open(2,FILE=fnum(2),status='unknown')
c
      CALL DATA
      ek=0.0d0;f=0.0d0;gk=0.0d0  !   CALL CLEAR
c
      NA=0 ! NA :number of integration point
ccc For all element
      DO 150 IE=1,NTE
         CALL GAUSS ! gauss integration point
         CALL DMAT ! elastic matrix 
ccc For all integration point
        DO 100 K=1,NGNG(IE)
         XXI  = AXI(K);ETA  = AETA(K);ZETA = AZETA(K); HNUM = H(K)
         NA=NA+1
         CALL INT  ! infinite element
c      write(99,777)ie,k,XXI,ETA,ZETA,HNUM,N(1)
c      write(99,777)ie,k,DNDXI(1),DNDETA(1),DNDZETA(1),HNUM,N(1)
c 777  format(2i3,5e12.3)
c         write(6,*)IE,K
         CALL JMAT   ! infinite element
         CALL BMAT   ! B matrix
         CALL BMMAT  ! B matrix for each integration point (NA)
         CALL BDBMAT ! BtDB matrix
         CALL GAUSS2 ! element stiffness matrix 
 100     CONTINUE
c
         CALL MERGE ! element stiffness --> global stiffness
         EK=0.0d0      !  clear element stiffness matrix 
c         write(6,*)ie
 150  CONTINUE
      write(6,*)'MERGE END'
      CALL BOUND  !   boundary condition
      write(6,*)'BOUND END'
c  CALL SOLVE(GK,F,U) ! gauss-erase-method
c  CALL SOLVE_C(GK,GKC,F,U,DR,ZM) ! cholesky method
c  CALL SOLVE_CD(GKC,F,U,DR,ZM) ! cholesky method(displacement)
      CALL SOLVE_C !  ICCG  method
      write(6,*)'SLOVE_C END'
      CALL SOLVE_CG ! ICCG  method
      write(6,*)'SLOVE_CG END'
      EP=0.0d0; SG=0.0d0    !  CLEAR
      CALL STRAIN  ! strain B*a
      CALL STRESS  ! stress D*strain
      CALL OUTPUT_AV ! output(averange at node)
      CALL OUTPUT ! output
      CLOSE(2)
c
c 7777 continue
c
      STOP
      END
