C     MAIN PROGRAM   with comment by izumi 1998-03-30
c----------------------------------------------------------------------
c  IELM  : Max. number of an element 
c  INODE : Max. number of a node
c  IRD   : Max. number of integration point in one element
c  IGK   : Max. size of stiffness matrix  INODE*IDIM
c  IBN   : Max. size of half band width
c  IBOUND: Max. number of constrain (force and displacement)
c  INA   : Max. number of integration point  IELM*IRD
c  INMTE : Max. number of a node in a element --> 8
c  INMNM : Max. size of element stiffness matrix  INMTE*IDIM
c  IMNUM : Max. number of the kind of material 
c  IDM   : DIMENSION
c  ICOM  : number of stress component (2-Dim->4 ,3Dim->6)
c----------------------------------------------------------------------
      PARAMETER (IELM=600,INODE=529,IRD=9,IGK=1058,IBN=50,IBOUND=100,
     +     INA=5400,INMTE=8,INMNM=16,IMNUM=10)
      PARAMETER (IDIM=2,ICOM=4)
c----------------------------------------------------------------------
c  DNDX1(i): dNi/dXI, DNDETA(i) : dNi/dETA
c  X(i) : x-coordinate of node i ,Y(i) : y-coordinate 
c----------------------------------------------------------------------
      DIMENSION DNDXI(INMTE),DNDETA(INMTE),X(INODE),Y(INODE),
c----------------------------------------------------------------------
c  B(i,j) : elemental stiffness matrix
c  AXI(i) : x-coordinate of i-th integration point ( -1 < AXI  < 1 )
c  AETA(i): y-coordinate of i-th integration point ( -1 < AETA < 1 )
c  H(i)   : weight parameter of i-th integration point
c  D(i,j) : elastic matrix (1-x,2-y,3-z,4-xy)
c----------------------------------------------------------------------
     +     B(ICOM,INMNM),AXI(IRD),AETA(IRD),H(IRD),D(ICOM,ICOM),
c----------------------------------------------------------------------
c  C(i,j)  : BT(INMNM,4)*D(4,4)*B(4,INMNM) Bt*D*B
c  EK(i,j) : element stiffness matrix 
c  NN(i,j) : node number of i-th local node of element j 
c----------------------------------------------------------------------
     +     C(INMNM,INMNM),EK(INMNM,INMNM),NN(INMTE,IELM),
c----------------------------------------------------------------------
c  GK(i,j) : global stiffness matrix  i: matrix size  j:half band width
c  F(i)    : force vector of node i
c  IFOR(i) : freedom number of i-th initial load point
c  FB(i)   : load value of i-th initial load point
c----------------------------------------------------------------------
     +     GK(IGK,IBN),F(IGK),IFOR(IBOUND),FB(IBOUND),
c----------------------------------------------------------------------
c  IDIS(i) : freedom number of i-th initial disp. point
c  UB(i)   : disp. value of i-th initial disp. point
c  U(i)    : displacement value of i-th freedom
c  EP(i,j) : i-th component strain at j-th integration point
c  BM(i)   : For strain (B matrix for i-th integration point)
c  SG(i,j) : i-th component stress at j-th integration point
c  BT(i)   : transposed B(4,INMNM) matrix 
c  BTD(i)  : BT(INMNM,4)*D(4,4)
c----------------------------------------------------------------------
     +     IDIS(IBOUND),UB(IBOUND),U(IGK),EP(ICOM,INA),
     +     BM(INA,ICOM,INMNM),SG(ICOM,INA),BT(INMNM,ICOM),
     +     BTD(INMNM,ICOM),
c----------------------------------------------------------------------
c  YO(i)   : Young modulus of i-th material 
c  PO(i)   : Poisson ratio of i-th material 
c  NOM(i)  : material number of i-th element
c  NMTE(i) : number of a node in i-th element
c  NGNG(i) : number of integration point in i-th element
c----------------------------------------------------------------------
     +     YO(IMNUM),PO(IMNUM),NOM(IELM),NMTE(IELM),NGNG(IELM)
c----------------------------------------------------------------------
c  N    : shape function Ni
c  JINV : Inverse of Jacobian matrix
c  J    : Jacobian matrix
c----------------------------------------------------------------------
      REAL N(INMTE),JINV(IDIM,IDIM),J(IDIM,IDIM)
c      
      OPEN(2,FILE='result.dat')
c--------------------------------------------------------------
c FOR SUBROUTIE DATA
c  NGAUSS : Order of GAUSS integration
c  NPARA  : Analysis condition (0:plain stress,1:plain strain)
c  T      : Thickness
c  NTN    : total number of nodes
c  NTE    : total number of elements
c  NTF    : total number of freedoms
c  IE     : element number(variables)
c  NTFOR  : total number of freedoms with initial load
c  NTDIS  : total number of freedoms with initial displacement
c  NNA    : The total number of integration points
c  MNUM   : number of the kind of material
c--------------------------------------------------------------
      CALL DATA(A,NMTE,NGAUSS,NPARA,YO,PO,T,NTN,NTE,NTF,
     +     NN,X,Y,IE,NTFOR,IFOR,FB,NTDIS,IDIS,UB,NGNG,NNA,
     +     INMTE,NOM,MNUM,IDIM)
c--- band width
c--------------------------------------------------------------
c  NB     : half band width
c--------------------------------------------------------------
      CALL BAND(NTE,NN,NB,INMTE,NMTE)
C
      CALL CHECK(INODE,IELM,IBN,NTN,NTE,NB)
C
C      CALL GAUSS(AXI,AETA,H,NMTE,NGAUSS)
      CALL CLEAR(EK,GK,NTF,NB,F,IGK,INMNM)
      NA=0
ccc For all element
      DO 150 IE=1,NTE
c--- gauss integration point
         CALL GAUSS(AXI,AETA,H,NMTE,NGAUSS,IE)
c--- elastic matrix 
         CALL DMAT(NPARA,YO,PO,D,NOM,IE) 
ccc For all integration point
         DO 100 K=1,NGNG(IE)
            XI=AXI(K)
            ETA=AETA(K)
            HNUM=H(K)
c--------------------------------------------------------------
c  NA     : through number of integration point
c--------------------------------------------------------------
            NA=NA+1
c--------------------------------------------------------------
c  RX     : real space x-coordinate of integration pointc 
c              for axial symmetry element
c--------------------------------------------------------------
            CALL INT(XI,ETA,N,DNDXI,DNDETA,NMTE,IE,RX,X,NN,INMTE)
c--- Jacobian matrix and its inverse matrix
c--------------------------------------------------------------
c  DETJ   : det[J]
c--------------------------------------------------------------
            CALL JMAT(X,Y,DNDXI,DNDETA,J,JINV,DETJ,IE,NN,NMTE,INMTE,IE)
c B matrix
            CALL BMAT(JINV,B,N,DNDXI,DNDETA,IE,NMTE,IE,RX)
c B matrix for each integration point (NA)
            CALL BMMAT(B,NA,BM,INA,NMTE,IE)
c BtDB matrix
            CALL BDBMAT(B,C,D,NMTE,INMNM,BT,BTD,IE)
c element stiffness matrix 
            CALL GAUSS2(C,EK,HNUM,NPARA,N,X,DETJ,NMTE,NN,IE,
     +           INMTE,INMNM)
 100     CONTINUE
c
         CALL GAUSS3(NPARA,NMTE,EK,T,INMNM,IE)
c element stiffness --> global stiffness
         CALL MERGE(IE,EK,NN,GK,IGK,IBN,NMTE,INMTE,INMNM)
c clear element stiffness matrix 
         CALL CLEAR2(EK,NMTE,INMNM)
 150  CONTINUE
c boundary condition
      CALL BOUND(NTFOR,IFOR,FB,F,IDIS,UB,GK,NTDIS,NB,IGK,NTF)
c gauss-erase-method
      CALL SOLVE(GK,NB,NTF,F,U,IGK)
c strain B*a
      CALL STRAIN(NTE,NN,U,EP,IE,BM,INA,NNA,INMTE,NMTE)
c stress D*strain
      CALL STRESS(D,EP,SG,NNA)
c output 
      CALL OUTPUT(NTF,U,EP,SG,NNA)
      
      CLOSE(2)
      
      STOP
      END
